# Juju charm for deploying content to Apache

## How to deploy the charm

Use this charm as a subordinate to a charm like apache2. It will download the
content and install it on the server.

### High level swift example:
    swift upload -v my-charmblobs my-website.tgz
    juju deploy apache2
    juju deploy content-fetcher
    juju add-relation apache2:juju-info content-fetcher:juju-info

## How to upgrade website content
    swift upload -v my-charmblobs my-website.tgz
    juju upgrade-charm content-fetcher

## Worked example with content
### Prepare website content
    mkdir ~/static-content
    cd ~/static-content
    cat <<-EOF > index.html
    <html><body><h1>Website v1</h1>
    </body></html>
    EOF
    tar zcf mywebsite.tgz *
    swift upload -v my-charmblobs mywebsite.tgz

### Create charm config
    mkdir ~/juju-config
    cd ~/juju-config
    cat <<EOF > mywebsite-vhost.cfg
    <VirtualHost *:80>
            ServerName {{servername}}
            ServerAdmin me@myemailaddress.com
            DocumentRoot /srv/{{servername}}/mywebsite/
        
            CustomLog /var/log/apache2/{{servername}}-access.log combined
            ErrorLog /var/log/apache2/{{servername}}-error.log
    
            <Directory /srv/{{servername}}/mywebsite/>
                    AllowOverride AuthConfig Limit
                    Options Indexes FollowSymLinks MultiViews
                    Order allow,deny
                    Allow from all
                    AddDefaultCharset UTF-8
            </Directory>
    </VirtualHost>
    EOF
    cat <<EOF > content-fetcher.yaml
    content-fetcher:
        src_type: swift
        username: $OS_USERNAME
        secret: $OS_PASSWORD
        archive_url: mywebsite.tgz
        auth_url: $OS_AUTH_URL
        region_name: $OS_REGION_NAME
        tenant_name: $OS_TENANT_NAME
        container_name: my-charmblobs
        dest_dir: /srv/mywebsite.mydomain/mywebsite/
    EOF

### Deploy Charms
    cd ~/juju-config
    juju bootstrap
    juju deploy apache2
    juju set apache2 "servername=mywebsite.mydomain"
    juju set apache2 "vhost_http_template=$(base64 mywebsite-vhost.cfg)"
    juju deploy --config content-fetcher.yaml content-fetcher
    juju add-relation apache2:juju-info content-fetcher:juju-info
    juju ssh apache2/0 "curl http://localhost"
    <html><body><h1>Website v1</h1>
    </body></html>

### Upgrade website content
    cd static-content/
    sed -i 's/v1/v2/' index.html 
    tar zcf mywebsite.tgz *
    swift upload -v my-charmblobs mywebsite.tgz
    juju upgrade-charm content-fetcher

## Restrictions

## Todo
