#!/usr/bin/env python
import os
import sys
import time

from charmhelpers.core.hookenv import (
    log,
    Hooks,
    config,
)

from charmhelpers.core.host import (
    lsb_release,
    rsync,
    mkdir,

)

from charmhelpers.fetch import (
    add_source,
    install_remote,
)

hooks = Hooks()
hook_dir = os.path.abspath(os.path.dirname(__file__))
charm_dir = os.path.dirname(hook_dir)
template_dir = os.path.join(charm_dir, 'templates')

@hooks.hook('install')
def install():
    print "Modules look after installing their own prereqs"

def setup_ssh(src_server):
    ssh_dir = os.path.join(os.environ['HOME'],'.ssh')
    ssh_config = os.path.join(ssh_dir,'config')
    ssh_config_tmpl = os.path.join(template_dir,'ssh_config.tmpl')
    if not os.path.exists(ssh_dir):
        os.mkdir(ssh_dir, mode=0700)
    charm_key = os.path.join(charm_dir, 'id_content_fetcher')
    installed_key = os.path.join(ssh_dir, 'id_content_fetcher')
    if os.path.isfile(charm_key):
        with open(charm_key, 'r') as content_file:
            private_key = content_file.read()
        with open(installed_key, 'wt') as content_file:
            os.chmod(installed_key, 0600)
            content_file.write(private_key)
    with open(ssh_config_tmpl, 'r') as f:
        template = f.read()
    temp_args = {
        'src_server': src_server,
        'key_file': installed_key,
    }
    open_mode = 'w'
    if os.path.isfile(ssh_config):
        for line in open(ssh_config):
            if "Host " + src_server in line:
                return
        open_mode = 'a'
    with open(ssh_config, open_mode) as f:
        os.chmod(ssh_config, 0644)
        f.write(template.format(**temp_args))

@hooks.hook('config-changed')
def config_changed():
    src_type = config().get('src_type')
    dst_dir = config().get('dest_dir')
    if src_type == 'swift':
        if lsb_release()['DISTRIB_CODENAME'] == 'precise':
            add_source('cloud:precise-updates/folsom', '5EDB1B62EC4926EA')
        import swift
        sc = swift.SwiftFetchHandler(config().get('auth_url'),
                                     config().get('username'),
                                     config().get('secret'),
                                     config().get('tenant_name'))
        archive_dir = sc.install(config().get('container_name'),
                                 config().get('archive_url'))
    elif src_type == 'bzr+ssh':
        setup_ssh()
        src_url = config().get('archive_url')
        archive_dir = install_remote(src_url)
    else:
        src_url = config().get('archive_url')
        archive_dir = install_remote(src_url)
    if os.path.exists(dst_dir):
        backup_postfix = time.strftime("%Y%m%d-%H%M%S", time.gmtime())
        backup_dir = dst_dir.rstrip('/') + backup_postfix
        log("Backing up old content to " + backup_dir)
        mkdir(backup_dir, perms=0755)
        rsync(dst_dir + '/', backup_dir)
    else:
        mkdir(dst_dir, perms=0755)
    log("Installing new content")
    rsync(archive_dir + '/', dst_dir)


if __name__ == "__main__":
    hooks.execute(sys.argv)
