import os
from charmhelpers.fetch import (
    UnhandledSource
)
from charmhelpers.core.host import mkdir
from charmhelpers.payload.archive import (
    extract,
)

try:
    import swiftclient
except ImportError:
    from charmhelpers.fetch import (
        apt_install,
        apt_update,
    )
    apt_update()
    apt_install("python-swiftclient")
    import swiftclient


class SwiftFetchHandler():
    """Handler for swift"""
    def __init__(self, aurl, user, password, tenant_name):
        self.authurl = aurl
        self.user = user
        self.key = password
        self.tenant = tenant_name

    def get_connection(self):
        return swiftclient.client.Connection(authurl=self.authurl,
                                             user=self.user,
                                             key=self.key,
                                             tenant_name=self.tenant,
                                             auth_version='2.0')

    def can_handle(self):
        try:
            sc = self.get_connection()
            sc.close()
            return True
        except:
            return False

    def download(self, src_container, src_filename, dest):
        try:
            sc = self.get_connection()
            headers, content = sc.get_object(src_container, src_filename)
            with open(dest, 'w') as dest_file:
                dest_file.write(content)
        except Exception as e:
            if os.path.isfile(dest):
                os.unlink(dest)
            raise e

    def install(self,  src_container, src_filename):
        dest_dir = os.path.join(os.environ.get('CHARM_DIR'), 'fetched')
        if not os.path.exists(dest_dir):
            mkdir(dest_dir, perms=0755)
        dld_file = os.path.join(dest_dir, src_container)
        try:
            self.download(src_container, src_filename, dld_file)
        except OSError as e:
            raise UnhandledSource(e.strerror)
        return extract(dld_file)
